#!/usr/bin/env python

from googleapiclient.discovery import build
from googleapiclient.errors import HttpError
from ratelimit import limits, sleep_and_retry

from ytscraper.helper.echo import echoe


def get_youtube_handle(api_key):
    """ Returns the YouTube Data API v3 handle.

    This method returns the handle for the YouTube Data API v3 
    in order to request data.

    Parameter
    ---------
    api_key: str
        An authorized API key. 
    """
    try:
        return build('youtube', 'v3', developerKey=api_key)
    except HttpError:
        echoe(""" There was an error while connecting to the YouTube API.
        Please check your API key.""")


@sleep_and_retry
@limits(3_000_000, 100)
def check_rate_limit():
    """ Sentinel method that limits the requests per 100 seconds. """
    pass


# TODO relevanceLanguage parameter
def get_search_videos(handle,
                      query,
                      maxResults,
                      regionCode='de',
                      safeSearch='none'):
    check_rate_limit()
    response = handle.search().list(
        q=query,
        part='id',
        type='video',
        safeSearch=safeSearch,  # none, moderate, strict
        regionCode=regionCode,
        maxResults=maxResults).execute()
    return list(map(lambda l: l['id']['videoId'], response['items']))


def get_related_videos(handle,
                       videoId,
                       maxResults,
                       regionCode='de',
                       safeSearch='none'):
    check_rate_limit()
    response = handle.search().list(
        relatedToVideoId=videoId,
        part='id',
        type='video',
        safeSearch=safeSearch,  # none, moderate, strict
        regionCode=regionCode,
        maxResults=maxResults).execute()
    return list(map(lambda l: l['id']['videoId'], response['items']))



#!/usr/bin/env python

import os

import click
import yaml

from ytscraper.helper.echo import warning
from ytscraper import APP_NAME



def update_config(config, options):
    """ Updates a configuration dictionary.

        Parameters
        ----------
        config: dict
            A dictionary that should be updated.
        options: dict
            Values that should be put into `config`.
    """
    for key, value in options.items():
        if value:
            config[key] = value


def load_config(config_path=None):
    r"""Processes and returns the user configuration.

    This function reads a YAML configuration file from either a provided
    path or the standard configuration directory and returns it.

    Parameters
    ----------
    config_path: str, optional
        The file path to the configuration file. If not specified, 
        the method tries to read the default system-specific 
        configuration folder.

    Returns
    -------
    dict
        Configuration dictionary.

    Notes
    -----
        The standard configuration directory is system specific:
        - Mac OS X: "~/Library/Application Support/YouTube Scraper"
        - Unix: "~/.config/youtube-scraper"
        - Win7: C:\Users\<user>\AppData\Roaming\YouTube Scraper
    """
    if config_path:
        config = yaml.safe_load(open(config_path))
    else:
        try:
            config_folder = click.get_app_dir(__APP_NAME__, roaming=True)
            config_path = os.path.join(config_folder, 'config.yml')
            config = yaml.safe_load(open(config_path))
        except FileNotFoundError:
            click.echo(warning + "Configuration file not found:")
            click.echo(warning + config_path)
            config = {}

    return config



import pathlib
from setuptools import find_namespace_packages, setup

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()

setup(
    name="yt-scraper",
    version="0.1.2",
    description="Command line utility querying the YouTube API v3.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/rattletat/yt-scraper",
    author="Michael Brauweiler",
    author_email="rattletat@posteo.net",
    license="GPLv3",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Console",
        "Intended Audience :: Science/Research",
        "Topic :: Multimedia :: Video",
        "Topic :: Scientific/Engineering :: Information Analysis",
    ],
    keywords="youtube scraping API",
    project_urls={
        "Source": "https://github.com/rattletat/yt-scraper",
        "Tracker": "https://github.com/rattletat/yt-scraper/issues",
    },
    packages=find_namespace_packages(exclude=["tests"]),
    include_package_data=True,
    install_requires=[
        "appdirs==1.4.3",
        "cachetools==4.0.0",
        "certifi==2020.4.5.1",
        "chardet==3.0.4",
        "click==7.1.1",
        "google-api-core==1.16.0",
        "google-api-python-client==1.8.0",
        "google-auth==1.13.1",
        "google-auth-httplib2==0.0.3",
        "googleapis-common-protos==1.51.0",
        "httplib2==0.17.1",
        "idna==2.9",
        "protobuf==3.11.3",
        "pyasn1==0.4.8",
        "pyasn1-modules==0.2.8",
        "pytz==2019.3",
        "pyyaml==5.3.1",
        "ratelimit==2.2.1",
        "requests==2.23.0",
        "rsa==4.0",
        "six==1.14.0",
        "uritemplate==3.0.1",
        "urllib3==1.25.8",
    ],
    entry_points={"console_scripts": ["ytscraper = ytscraper.__main__:main"]},
)



#!/usr/bin/env python

from typing import NamedTuple, Tuple

from ytscraper.helper.yt_api import get_related_videos

class VideoNode(NamedTuple):
    videoId: str
    depth: int
    rank: int
    relatedVideos: Tuple

# TODO make it more obvious that number is only a maximal value
def build_node(handle, video_id, rank, depth, number):
    related_videos = get_related_videos(handle, video_id, number)
    video_node = VideoNode(video_id, depth, rank, tuple(related_videos))
    return video_node


def build_node_list(handle, video_ids, depth, number):
    video_nodes = []
    for rank, video_id in enumerate(video_ids):
        node = build_node(handle, video_id, rank, depth, number) 
        video_nodes.append(node)
    return video_nodes

